# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{KatakanaOrHiragana} (and fuzzy permutations)
# 
# Meaning: Script 'Katakana_Or_Hiragana'
#
return <<'END';
3031	3035	Katakana_Or_Hiragana
309B	309C	Katakana_Or_Hiragana
FF70		Katakana_Or_Hiragana
FF9E	FF9F	Katakana_Or_Hiragana
END
