# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Ethiopic} (and fuzzy permutations)
# 
# Meaning: Script 'Ethiopic'
#
return <<'END';
1200	1206	Ethiopic
1208	1246	Ethiopic
1248		Ethiopic
124A	124D	Ethiopic
1250	1256	Ethiopic
1258		Ethiopic
125A	125D	Ethiopic
1260	1286	Ethiopic
1288		Ethiopic
128A	128D	Ethiopic
1290	12AE	Ethiopic
12B0		Ethiopic
12B2	12B5	Ethiopic
12B8	12BE	Ethiopic
12C0		Ethiopic
12C2	12C5	Ethiopic
12C8	12CE	Ethiopic
12D0	12D6	Ethiopic
12D8	12EE	Ethiopic
12F0	130E	Ethiopic
1310		Ethiopic
1312	1315	Ethiopic
1318	131E	Ethiopic
1320	1346	Ethiopic
1348	135A	Ethiopic
1361	137C	Ethiopic
END
