# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Sm}
# 	\p{Sm} (and fuzzy permutations)
# 
# Meaning: General Category 'Sm'
#
return <<'END';
002B		
003C	003E	
007C		
007E		
00AC		
00B1		
00D7		
00F7		
03F6		
2044		
2052		
207A	207C	
208A	208C	
2140	2144	
214B		
2190	2194	
219A	219B	
21A0		
21A3		
21A6		
21AE		
21CE	21CF	
21D2		
21D4		
21F4	22FF	
2308	230B	
2320	2321	
237C		
239B	23B3	
25B7		
25C1		
25F8	25FF	
266F		
27D0	27E5	
27F0	27FF	
2900	2982	
2999	29D7	
29DC	29FB	
29FE	2AFF	
FB29		
FE62		
FE64	FE66	
FF0B		
FF1C	FF1E	
FF5C		
FF5E		
FFE2		
FFE9	FFEC	
1D6C1		
1D6DB		
1D6FB		
1D715		
1D735		
1D74F		
1D76F		
1D789		
1D7A9		
1D7C3		
END
