# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Nd}
# 	\p{Nd} (and fuzzy permutations)
# 
# Meaning: General Category 'Nd'
#
return <<'END';
0030	0039	
0660	0669	
06F0	06F9	
0966	096F	
09E6	09EF	
0A66	0A6F	
0AE6	0AEF	
0B66	0B6F	
0BE7	0BEF	
0C66	0C6F	
0CE6	0CEF	
0D66	0D6F	
0E50	0E59	
0ED0	0ED9	
0F20	0F29	
1040	1049	
1369	1371	
17E0	17E9	
1810	1819	
1946	194F	
FF10	FF19	
104A0	104A9	
1D7CE	1D7FF	
END
