# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{BidiBN}
# 
# Meaning: Bi-directional category 'BN'
#
return <<'END';
0000	0008	
000E	001B	
007F	0084	
0086	009F	
00AD		
070F		
200B	200D	
2060	2063	
206A	206F	
FEFF		
1D173	1D17A	
E0001		
E0020	E007F	
END
