# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Sterm} (and fuzzy permutations)
# 
# Meaning: Extended property 'STerm'
#
return <<'END';
0021		STerm
002E		STerm
003F		STerm
055C		STerm
055E		STerm
0589		STerm
061F		STerm
06D4		STerm
0700	0702	STerm
0964	0965	STerm
104A	104B	STerm
1362		STerm
1367	1368	STerm
166E		STerm
1803		STerm
1809		STerm
1944	1945	STerm
203C	203D	STerm
2047	2049	STerm
3002		STerm
FE52		STerm
FE56	FE57	STerm
FF01		STerm
FF0E		STerm
FF1F		STerm
FF61		STerm
END
