# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Yi} (and fuzzy permutations)
# 
# Meaning: Script 'Yi'
#
return <<'END';
A000	A48C	Yi
A490	A4C6	Yi
END
