# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{DCnoBreak}
# 
# Meaning: Compatible with 'noBreak'
#
return <<'END';
00A0		
0F0C		
2007		
2011		
202F		
END
