# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Hebrew} (and fuzzy permutations)
# 
# Meaning: Script 'Hebrew'
#
return <<'END';
0591	05A1	Hebrew
05A3	05B9	Hebrew
05BB	05C4	Hebrew
05D0	05EA	Hebrew
05F0	05F4	Hebrew
FB1D	FB36	Hebrew
FB38	FB3C	Hebrew
FB3E		Hebrew
FB40	FB41	Hebrew
FB43	FB44	Hebrew
FB46	FB4F	Hebrew
END
