# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{BidiEN}
# 
# Meaning: Bi-directional category 'EN'
#
return <<'END';
0030	0039	
00B2	00B3	
00B9		
06F0	06F9	
2070		
2074	2079	
2080	2089	
2488	249B	
FF10	FF19	
1D7CE	1D7FF	
END
