# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{DCvertical}
# 
# Meaning: Compatible with 'vertical'
#
return <<'END';
309F		
30FF		
FE30	FE44	
FE47	FE48	
END
