# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Cc}
# 	\p{Cc} (and fuzzy permutations)
# 
# Meaning: General Category 'Cc'
#
return <<'END';
0000	001F	
007F	009F	
END
