# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Latin} (and fuzzy permutations)
# 
# Meaning: Script 'Latin'
#
return <<'END';
0041	005A	Latin
0061	007A	Latin
00AA		Latin
00BA		Latin
00C0	00D6	Latin
00D8	00F6	Latin
00F8	0236	Latin
0250	02B8	Latin
02E0	02E4	Latin
1D00	1D25	Latin
1D2C	1D5C	Latin
1D62	1D65	Latin
1D6B		Latin
1E00	1E9B	Latin
1EA0	1EF9	Latin
2071		Latin
207F		Latin
212A	212B	Latin
FB00	FB06	Latin
FF21	FF3A	Latin
FF41	FF5A	Latin
END
