# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Tamil} (and fuzzy permutations)
# 
# Meaning: Script 'Tamil'
#
return <<'END';
0B82	0B83	Tamil
0B85	0B8A	Tamil
0B8E	0B90	Tamil
0B92	0B95	Tamil
0B99	0B9A	Tamil
0B9C		Tamil
0B9E	0B9F	Tamil
0BA3	0BA4	Tamil
0BA8	0BAA	Tamil
0BAE	0BB5	Tamil
0BB7	0BB9	Tamil
0BBE	0BC2	Tamil
0BC6	0BC8	Tamil
0BCA	0BCD	Tamil
0BD7		Tamil
0BE7	0BFA	Tamil
END
