# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{BidiWS}
# 
# Meaning: Bi-directional category 'WS'
#
return <<'END';
000C		
0020		
1680		
180E		
2000	200A	
2028		
202F		
205F		
3000		
END
