# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{LinearB} (and fuzzy permutations)
# 
# Meaning: Script 'Linear_B'
#
return <<'END';
10000	1000B	Linear_B
1000D	10026	Linear_B
10028	1003A	Linear_B
1003C	1003D	Linear_B
1003F	1004D	Linear_B
10050	1005D	Linear_B
10080	100FA	Linear_B
END
