# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{No}
# 	\p{No} (and fuzzy permutations)
# 
# Meaning: General Category 'No'
#
return <<'END';
00B2	00B3	
00B9		
00BC	00BE	
09F4	09F9	
0BF0	0BF2	
0F2A	0F33	
1372	137C	
17F0	17F9	
2070		
2074	2079	
2080	2089	
2153	215F	
2460	249B	
24EA	24FF	
2776	2793	
3192	3195	
3220	3229	
3251	325F	
3280	3289	
32B1	32BF	
10107	10133	
10320	10323	
END
