# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Armenian} (and fuzzy permutations)
# 
# Meaning: Script 'Armenian'
#
return <<'END';
0531	0556	Armenian
0559	055F	Armenian
0561	0587	Armenian
058A		Armenian
FB13	FB17	Armenian
END
