# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Inherited} (and fuzzy permutations)
# 
# Meaning: Script 'Inherited'
#
return <<'END';
0300	0357	Inherited
035D	036F	Inherited
064B	0655	Inherited
0670		Inherited
200C	200D	Inherited
20D0	20EA	Inherited
302A	302F	Inherited
3099	309A	Inherited
FE00	FE0F	Inherited
FE20	FE23	Inherited
1D167	1D169	Inherited
1D17B	1D182	Inherited
1D185	1D18B	Inherited
1D1AA	1D1AD	Inherited
E0100	E01EF	Inherited
END
