# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Lm}
# 	\p{Lm} (and fuzzy permutations)
# 
# Meaning: General Category 'Lm'
#
return <<'END';
02B0	02C1	
02C6	02D1	
02E0	02E4	
02EE		
037A		
0559		
0640		
06E5	06E6	
0E46		
0EC6		
17D7		
1843		
1D2C	1D61	
3005		
3031	3035	
303B		
309D	309E	
30FC	30FE	
FF70		
FF9E	FF9F	
END
