# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Arabic} (and fuzzy permutations)
# 
# Meaning: Script 'Arabic'
#
return <<'END';
060D	0615	Arabic
0621	063A	Arabic
0641	064A	Arabic
0656	0658	Arabic
066A	066F	Arabic
0671	06DC	Arabic
06DE	06FF	Arabic
FB50	FBB1	Arabic
FBD3	FD3D	Arabic
FD50	FD8F	Arabic
FD92	FDC7	Arabic
FDF0	FDFC	Arabic
FE70	FE74	Arabic
FE76	FEFC	Arabic
END
