# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Limbu} (and fuzzy permutations)
# 
# Meaning: Script 'Limbu'
#
return <<'END';
1900	191C	Limbu
1920	192B	Limbu
1930	193B	Limbu
1940		Limbu
1944	194F	Limbu
END
