# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Ps}
# 	\p{Ps} (and fuzzy permutations)
# 
# Meaning: General Category 'Ps'
#
return <<'END';
0028		
005B		
007B		
0F3A		
0F3C		
169B		
201A		
201E		
2045		
207D		
208D		
2329		
23B4		
2768		
276A		
276C		
276E		
2770		
2772		
2774		
27E6		
27E8		
27EA		
2983		
2985		
2987		
2989		
298B		
298D		
298F		
2991		
2993		
2995		
2997		
29D8		
29DA		
29FC		
3008		
300A		
300C		
300E		
3010		
3014		
3016		
3018		
301A		
301D		
FD3E		
FE35		
FE37		
FE39		
FE3B		
FE3D		
FE3F		
FE41		
FE43		
FE47		
FE59		
FE5B		
FE5D		
FF08		
FF3B		
FF5B		
FF5F		
FF62		
END
