# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Gujarati} (and fuzzy permutations)
# 
# Meaning: Script 'Gujarati'
#
return <<'END';
0A81	0A83	Gujarati
0A85	0A8D	Gujarati
0A8F	0A91	Gujarati
0A93	0AA8	Gujarati
0AAA	0AB0	Gujarati
0AB2	0AB3	Gujarati
0AB5	0AB9	Gujarati
0ABC	0AC5	Gujarati
0AC7	0AC9	Gujarati
0ACB	0ACD	Gujarati
0AD0		Gujarati
0AE0	0AE3	Gujarati
0AE6	0AEF	Gujarati
0AF1		Gujarati
END
