# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{InSupplementalArrowsA} (and fuzzy permutations)
# 
# Meaning: Block 'Supplemental Arrows-A'
#
return <<'END';
27F0	27FF	Supplemental Arrows-A
END
