# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Common} (and fuzzy permutations)
# 
# Meaning: Script 'Common'
#
return <<'END';
0000	0040	Common
005B	0060	Common
007B	00A9	Common
00AB	00B9	Common
00BB	00BF	Common
00D7		Common
00F7		Common
02B9	02DF	Common
02E5	02FF	Common
037E		Common
0387		Common
0589		Common
0600	0603	Common
060C		Common
061B		Common
061F		Common
0640		Common
0660	0669	Common
06DD		Common
0964	0965	Common
0970		Common
0E3F		Common
10FB		Common
16EB	16ED	Common
1735	1736	Common
2000	200B	Common
200E	2054	Common
2057		Common
205F	2063	Common
206A	2070	Common
2074	207E	Common
2080	208E	Common
20A0	20B1	Common
2100	2125	Common
2127	2129	Common
212C	213B	Common
213D	214B	Common
2153	2183	Common
2190	23D0	Common
2400	2426	Common
2440	244A	Common
2460	2617	Common
2619	267D	Common
2680	2691	Common
26A0	26A1	Common
2701	2704	Common
2706	2709	Common
270C	2727	Common
2729	274B	Common
274D		Common
274F	2752	Common
2756		Common
2758	275E	Common
2761	2794	Common
2798	27AF	Common
27B1	27BE	Common
27D0	27EB	Common
27F0	27FF	Common
2900	2B0D	Common
2FF0	2FFB	Common
3000	3004	Common
3006		Common
3008	3020	Common
3030		Common
3036	3037	Common
303C	303F	Common
30A0		Common
30FB	30FC	Common
3190	319F	Common
3220	3243	Common
3250	325F	Common
327F	32FE	Common
3300	33FF	Common
4DC0	4DFF	Common
E000	F8FF	Common
FD3E	FD3F	Common
FDFD		Common
FE30	FE52	Common
FE54	FE66	Common
FE68	FE6B	Common
FEFF		Common
FF01	FF20	Common
FF3B	FF40	Common
FF5B	FF65	Common
FFE0	FFE6	Common
FFE8	FFEE	Common
FFF9	FFFD	Common
10100	10102	Common
10107	10133	Common
10137	1013F	Common
1D000	1D0F5	Common
1D100	1D126	Common
1D12A	1D166	Common
1D16A	1D17A	Common
1D183	1D184	Common
1D18C	1D1A9	Common
1D1AE	1D1DD	Common
1D300	1D356	Common
1D400	1D454	Common
1D456	1D49C	Common
1D49E	1D49F	Common
1D4A2		Common
1D4A5	1D4A6	Common
1D4A9	1D4AC	Common
1D4AE	1D4B9	Common
1D4BB		Common
1D4BD	1D4C3	Common
1D4C5	1D505	Common
1D507	1D50A	Common
1D50D	1D514	Common
1D516	1D51C	Common
1D51E	1D539	Common
1D53B	1D53E	Common
1D540	1D544	Common
1D546		Common
1D54A	1D550	Common
1D552	1D6A3	Common
1D6A8	1D7C9	Common
1D7CE	1D7FF	Common
E0001		Common
E0020	E007F	Common
F0000	FFFFD	Common
100000	10FFFD	Common
END
