# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Mongolian} (and fuzzy permutations)
# 
# Meaning: Script 'Mongolian'
#
return <<'END';
1800	180E	Mongolian
1810	1819	Mongolian
1820	1877	Mongolian
1880	18A9	Mongolian
END
