# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Tagalog} (and fuzzy permutations)
# 
# Meaning: Script 'Tagalog'
#
return <<'END';
1700	170C	Tagalog
170E	1714	Tagalog
END
