# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

return <<'END';
1100		G
1101		GG
1102		N
1103		D
1104		DD
1105		R
1106		M
1107		B
1108		BB
1109		S
110A		SS
110B		
110C		J
110D		JJ
110E		C
110F		K
1110		T
1111		P
1112		H
1161		A
1162		AE
1163		YA
1164		YAE
1165		EO
1166		E
1167		YEO
1168		YE
1169		O
116A		WA
116B		WAE
116C		OE
116D		YO
116E		U
116F		WEO
1170		WE
1171		WI
1172		YU
1173		EU
1174		YI
1175		I
11A8		G
11A9		GG
11AA		GS
11AB		N
11AC		NJ
11AD		NH
11AE		D
11AF		L
11B0		LG
11B1		LM
11B2		LB
11B3		LS
11B4		LT
11B5		LP
11B6		LH
11B7		M
11B8		B
11B9		BS
11BA		S
11BB		SS
11BC		NG
11BD		J
11BE		C
11BF		K
11C0		T
11C1		P
11C2		H
END
