# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Myanmar} (and fuzzy permutations)
# 
# Meaning: Script 'Myanmar'
#
return <<'END';
1000	1021	Myanmar
1023	1027	Myanmar
1029	102A	Myanmar
102C	1032	Myanmar
1036	1039	Myanmar
1040	1059	Myanmar
END
