# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Sc}
# 	\p{Sc} (and fuzzy permutations)
# 
# Meaning: General Category 'Sc'
#
return <<'END';
0024		
00A2	00A5	
09F2	09F3	
0AF1		
0BF9		
0E3F		
17DB		
20A0	20B1	
FDFC		
FE69		
FF04		
FFE0	FFE1	
FFE5	FFE6	
END
