# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{BidiCS}
# 
# Meaning: Bi-directional category 'CS'
#
return <<'END';
002C		
002E	002F	
003A		
00A0		
060C		
2044		
FE50		
FE52		
FE55		
FF0C		
FF0E		
FF1A		
END
