# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Tibetan} (and fuzzy permutations)
# 
# Meaning: Script 'Tibetan'
#
return <<'END';
0F00	0F47	Tibetan
0F49	0F6A	Tibetan
0F71	0F8B	Tibetan
0F90	0F97	Tibetan
0F99	0FBC	Tibetan
0FBE	0FCC	Tibetan
0FCF		Tibetan
END
